@@ -1,9 +1,13 @@
 from decimal import Decimal, getcontext
 from itertools import takewhile, repeat
 from functools import wraps
+from uuid import uuid4
+from math import floor
+from decimal import *
 import numpy as np
 import datetime
 import logging
+import socket
 import fnmatch
 import random
 import psutil
@@ -801,6 +805,23 @@ class Handler(object):
         if self.file_exists():
             os.remove(self.filename)
 
+    def array2d_to_file(self,
+                        array,
+                        sep=",",
+                        action="w"):
+        """
+        Method to write a numpy array to text file
+        :param array: Numpy ndarray
+        :param sep: Separator between array items for text output.
+                    If "" (empty), a binary file is written, equivalent to file.write(a.tobytes())
+        :param action: "w" for write, "a" for append, "r" for read
+        :return: None
+        """
+        with open(self.filename, action) as f:
+            for row in array:
+                row_str = sep.join([str(elem) for elem in row]) + '\n'
+                f.write(row_str)
+
     def copy_file(self,
                   other_dir,
                   replace=True,
@@ -1259,6 +1280,15 @@ class Handler(object):
                      append=False,
                      header=True):
 
+        """
+        Method to write to a csv file
+        :param list_of_dicts: list of dictionaries to be written
+        :param outfile: output file name
+        :param delimiter: Delimiter examples : ,  ; |
+        :param append: If the output should be appended to an already exising file
+        :param header: If the header should be written
+        """
+
         if outfile is None:
             raise ValueError("No file name for writing")
 
@@ -1267,7 +1297,8 @@ class Handler(object):
 
         lines = list()
         if header:
-            lines.append(delimiter.join(list(list_of_dicts[0])))
+            lines.append(delimiter.join([str(elem) for elem in list(list_of_dicts[0])]))
+
         for data_dict in list_of_dicts:
             lines.append(delimiter.join(list(str(val) for _, val in data_dict.items())))
 
@@ -1371,6 +1402,7 @@ class Opt:
         Instantiate Opt class
         """
         self.obj = obj  # mutable object
+        self.hex = uuid4().get_hex()
 
     def __repr__(self):
         return "<Optional notice class>"
@@ -1439,6 +1471,272 @@ class Opt:
         return 'T' + datetime.datetime.now().isoformat().replace(':', '')\
             .replace('.', '').replace('-', '').replace('T', '') + '.tmp'
 
+    @staticmethod
+    def lists_to_dict(value_list=None,
+                      column_list=None):
+        """
+        Constructs a dictionary from two lists
+        :param value_list: list of values in a dictionary
+        :param column_list: list of dictionary keys
+        :return: Dictionary
+        """
+        if column_list is not None:
+            if len(value_list) != len(column_list):
+                raise ValueError("Returned query does not match column list")
+            else:
+                return dict(zip(column_list,
+                                value_list))
+
+        else:
+            return dict(zip(['column' + str(i + 1) for i in range(0, len(value_list))],
+                            value_list))
+
+    @staticmethod
+    def dtype(input_data):
+
+        """
+        Method to output data type as string
+        :param input_data: input data
+        :return: string
+        """
+        try:
+            val = type(input_data).__name__
+        except (ValueError, SyntaxError):
+            val = None
+
+        return val
+
+
+
+    @staticmethod
+    def ogr_geom_type(input,
+                      format='num'):
+        """
+        Method to return OGR geometry type from input string
+        :param input: String to convert to OGR geometry type code
+        :param format: Which format should be output? (default: 'num'; options: 'num', 'name')
+        :return: OGR geometry type code
+        """
+
+        if format == 'num':
+            comp_str = input.lower()
+            comp_dict = {
+                'point': 1,
+                'line': 2,
+                'polygon': 3,
+                'multipoint': 4,
+                'multilinestring': 5,
+                'multipolygon': 6,
+                'geometry': 0,
+                'no geometry': 100
+            }
+            try:
+                return comp_dict[comp_str]
+            except (KeyError, NameError):
+                return None
+
+        elif format == 'name':
+            comp_dict = {
+                1: 'point',
+                2: 'line',
+                3: 'polygon',
+                4: 'multipoint',
+                5: 'multilinestring',
+                6: 'multipolygon',
+                0: 'geometry',
+                100: 'no geometry',
+            }
+            try:
+                return comp_dict[input].upper()
+            except (KeyError, NameError):
+                return None
+
+        else:
+            raise(ValueError('Invalid format'))
+
+    @staticmethod
+    def list_size(query_list):
+        """
+        Find size of a list object even if it is a one element non-list
+        :param query_list:
+        """
+
+        if isinstance(query_list, list):
+            return len(query_list)
+        else:
+            return 1
+
+    @staticmethod
+    def remove_strings(main_str, remove_str_list):
+        """
+        Remove a list of strings from a given string
+        :param main_str: Input string
+        :param remove_str_list: List of strings to be removed
+        :return:
+        """
+        temp = ''
+        for i in range(0, len(remove_str_list)):
+
+            # split the strings
+            temp_strings = main_str.split(remove_str_list[i])
+
+            # remove the string and rejoin
+            if remove_str_list[i] != '\n':
+                temp = ''.join(list(string for string in temp_strings))
+            else:
+                temp = '\n'.join(list(string for string in temp_strings if string != ''))
+
+            # reassign
+            main_str = temp
+
+        return temp
+
+    @staticmethod
+    def hostname():
+        """
+        Method to print machine hostname
+        :return: String
+        """
+        return socket.gethostname()
+
+    @staticmethod
+    def copy_item(item):
+        return copy.deepcopy(item)
+
+    @staticmethod
+    def growth_curve(n,
+                     maxn=100,
+                     mi=8):
+        """
+        Method to emulate growth curve. This is used to determine
+        number of compute nodes as number of operations increase
+        :param n: Number of operations required
+        :param maxn: Maximum value of output
+        :param mi: A coefficient (default: 8)
+        :return: Integer
+        """
+        x = n
+        mx = maxn
+        y = mx * (lambda t: (1 + mx ** (-1.25 * t / mi - 0.5)) **
+                            (-1.0))((float(2.0 * x - mi * mx)) / mx)
+        return int(np.floor(y))
+
+
+
+    @staticmethod
+    def ymd_to_yday(ymd_string):
+        """
+        Method to return tuple of (julian day, year) from date of format YYYY-MM-DD
+        :param ymd_string: String with date format YYYY-MM-DD
+        :return: Tuple (julian day, year)
+        """
+
+        date = datetime.datetime.strptime(ymd_string, '%Y-%m-%d')
+        tt = date.timetuple()
+        return tt.tm_yday, date.year
+
+    @staticmethod
+    def is_leap(year):
+        """
+        Method to determine if a year is leap or not
+        :param year: Year
+        :return: Boolean (True or False)
+        """
+        return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)
+
+    @staticmethod
+    def get_date_ranges(start_date, end_date):
+        """
+        Method to convert date string of the format YYYY-MM-DD to list of julian day ranges per year
+        :param start_date: date of format YYYY-MM-DD
+        :param end_date: date of format YYYY-MM-DD
+        :return: List of dictionaries of format {'start': 150, 'end': 210 , 'year': 2017 }
+        """
+
+        start_jday, start_year = Opt.ymd_to_yday(start_date)
+        end_jday, end_year = Opt.ymd_to_yday(end_date)
+
+        n_years = end_year - start_year + 1
+
+        if n_years == 1:
+            dates = [{'start': start_jday,
+                      'end': end_jday,
+                      'year': start_year}]
+
+        elif n_years > 1:
+            dates = list()
+
+            start_dict = {'start': start_jday,
+                          'end': 365,
+                          'year': start_year}
+
+            if Opt.is_leap(start_year):
+                start_dict['end'] = 366
+
+            dates.append(start_dict)
+
+            for i in range(0, n_years - 2):
+
+                temp_dict = {'start': 1,
+                             'end': 365,
+                             'year': start_year + i + 1}
+
+                if Opt.is_leap(start_year + i + 1):
+                    temp_dict['end'] = 366
+
+                dates.append(temp_dict)
+
+            end_dict = {'start': 1,
+                        'end': end_jday,
+                        'year': end_year}
+
+            if Opt.is_leap(end_year):
+                end_dict['end'] = 366
+
+            dates.append(end_dict)
+
+        else:
+            raise ValueError("Invalid dates or bad count")
+
+        return dates
+
+    @staticmethod
+    def custom_list(start,
+                    end,
+                    step=None):
+        """
+        List with custom first number but rest all follow same increment; integers only
+        :param start: starting integer
+        :param end: ending integer
+        :param step: step integer
+        :return: list of integers
+        """
+
+        # end of list
+        end = end + step - (end % step)
+
+        # initialize the list
+        out = list()
+
+        # make iterator
+        if step is not None:
+            if start % step > 0:
+                out.append(start)
+                if start < step:
+                    start = step
+                elif start > step:
+                    start = start + step - (start % step)
+            iterator = range(start, end, step)
+        else:
+            step = 1
+            iterator = range(start, step, end)
+
+        # add the rest of the list to out
+        for i in iterator:
+            out.append(i)
+
+        return out
+
 
 class FTPHandler(Handler):
     """Class to handle remote IO for ftp connection"""
