@@ -413,6 +413,90 @@ class Vector(object):
 
             return Vector.ogr_data_type(converted_inp, _return)
 
+    @staticmethod
+    def sizeof_ogr_datatype(type_number):
+        """
+        Method to determine the byte size of data types stored in CloudSQL
+        :param type_number: Data type code for Raster tile
+        :return: Size in bytes
+        """
+
+        if type_number == 1:
+            return 1
+        elif type_number == 2:
+            return 2
+        elif type_number == 3:
+            return 2
+        elif type_number == 4:
+            return 4
+        elif type_number == 5:
+            return 4
+        elif type_number == 6:
+            return 4
+        elif type_number == 7:
+            return 8
+        else:
+            raise ValueError("Unknown datatype code")
+
+    @staticmethod
+    def utm_zone_to_epsg(zone,
+                         hemisphere=None,
+                         tile=None):
+        """
+        Method to return EPSG SRID for UTM zone
+        :param zone: UTM zone
+        :param hemisphere: North or South hemishpere
+        :param tile: Tile IDs (ABCDEFGHJKLM for Northern hemisphere and NPQRSTUVWXYZ for southern)
+        :return: EPSG SRID with WGS84 datum
+        """
+        if hemisphere is not None:
+            if hemisphere == 'N':
+                return int('326' + str(zone).zfill(2))
+            elif hemisphere == 'S':
+                return int('327' + str(zone).zfill(2))
+            else:
+                raise ValueError('Invalid hemisphere label')
+        elif tile is not None:
+            if tile in 'NPQRSTUVWXYZ':
+                return int('326' + str(zone).zfill(2))
+            elif tile in 'ABCDEFGHJKLM':
+                return int('327' + str(zone).zfill(2))
+            else:
+                raise ValueError('Invalid tile label')
+
+    @staticmethod
+    def utm_zone_to_proj4(zone,
+                          hemisphere=None,
+                          tile=None,
+                          datum='WGS84'):
+        """
+        Method to return PROJ4 string for UTM zone
+        :param zone: UTM zone
+        :param hemisphere: North or South hemishpere
+        :param tile: Tile IDs (ABCDEFGHJKLM for Northern hemisphere and NPQRSTUVWXYZ for southern)
+        :param datum: Name of the datum used in the projection
+        :return: PROJ4 string
+        """
+        if hemisphere is not None:
+            if hemisphere == 'N':
+                return '+proj=utm +zone={z} +datum={d}'.format(z=str(zone),
+                                                               d=datum)
+            elif hemisphere == 'S':
+                return '+proj=utm +zone={z} +datum={d} +south'.format(z=str(zone),
+                                                                      d=datum)
+            else:
+                raise ValueError('Invalid hemisphere label')
+
+        elif tile is not None:
+            if tile in 'NPQRSTUVWXYZ':
+                return '+proj=utm +zone={z} +datum={d}'.format(z=str(zone),
+                                                               d=datum)
+            elif tile in 'ABCDEFGHJKLM':
+                return '+proj=utm +zone={z} +datum={d} +south'.format(z=str(zone),
+                                                                      d=datum)
+            else:
+                raise ValueError('Invalid tile label')
+
     @staticmethod
     def wkt_from_coords(coords,
                         geom_type='point'):
@@ -574,6 +658,15 @@ class Vector(object):
         if remove:
             vector = None
 
+    @staticmethod
+    def to_json(wkt_string):
+        """
+        Method to convert wkt string to json
+        :return: string
+        """
+        geom = ogr.CreateGeometryFromWkt(wkt_string)
+        return geom.ExportToJson().replace('"', "'")
+
     def write_vector(self,
                      outfile=None,
                      in_memory=False):
